@@ -19,6 +19,9 @@ import sqlite3
 import json
 import cravat.cravat_util as cu
 import cravat.admin_util as au
+import re
+from types import SimpleNamespace
+import cravat.util
 
 class BaseAnnotator(object):
 
@@ -31,13 +34,31 @@ class BaseAnnotator(object):
                              'crg':[x['name'] for x in crg_def]}
     required_conf_keys = ['level', 'output_columns']
 
-    def __init__(self, cmd_args, status_writer, live=False):
+    def __init__(self, *inargs, **inkwargs):
         try:
+            main_fpath = os.path.abspath(sys.modules[self.__module__].__file__)
+            self.primary_input_path = None
+            self.secondary_paths = None
+            self.output_dir = None
+            self.output_basename = None
+            self.plain_output = None
+            self.job_conf_path = None
+            self.logger = None
+            self.dbconn = None
+            self.cursor = None
+            self._define_cmd_parser()
+            self.args = cravat.util.get_args(self.cmd_arg_parser, inargs, inkwargs)
+            self.parse_cmd_args(inargs, inkwargs)
+            if hasattr(self.args, 'status_writer') == False:
+                self.status_writer = None
+            else:
+                self.status_writer = self.args.status_writer
+            if hasattr(self.args, 'live') == False:
+                live = False
+            else:
+                live = self.args.live
             if live:
                 return
-            self.status_writer = status_writer
-            self.logger = None
-            main_fpath = cmd_args[0]
             main_basename = os.path.basename(main_fpath)
             if '.' in main_basename:
                 self.module_name = '.'.join(main_basename.split('.')[:-1])
@@ -47,14 +68,7 @@ class BaseAnnotator(object):
             self.module_dir = os.path.dirname(main_fpath)
             self.annotator_dir = os.path.dirname(main_fpath)
             self.data_dir = os.path.join(self.module_dir, 'data')
-            self.primary_input_path = None
-            self.secondary_paths = None
-            self.output_dir = None
-            self.output_basename = None
-            self.plain_output = None
-            self.job_conf_path = None
             # Load command line opts
-            self.parse_cmd_args(cmd_args)
             self._setup_logger()
             config_loader = ConfigLoader(self.job_conf_path)
             self.conf = config_loader.get_module_conf(self.module_name)
@@ -70,8 +84,6 @@ class BaseAnnotator(object):
                 self.annotator_version = self.conf['version']
             else:
                 self.annotator_version = ''
-            self.dbconn = None
-            self.cursor = None
         except Exception as e:
             self._log_exception(e)
 
@@ -123,7 +135,7 @@ class BaseAnnotator(object):
                                 help='Secondary inputs. '\
                                      +'Format as <module_name>:<path>')
             parser.add_argument('-n',
-                                dest='name',
+                                dest='run_name',
                                 help='Name of job. Default is input file name.')
             parser.add_argument('-d',
                                 dest='output_dir',
@@ -140,60 +152,69 @@ class BaseAnnotator(object):
                 dest='confs',
                 default='{}',
                 help='Configuration string')
+            parser.add_argument('--silent',
+                dest='silent',
+                default=False,
+                help='Silent operation')
             self.cmd_arg_parser = parser
         except Exception as e:
             self._log_exception(e)
 
     # Parse the command line arguments
-    def parse_cmd_args(self, cmd_args):
+    def parse_cmd_args(self, inargs, inkwargs):
         try:
-            self._define_cmd_parser()
-            parsed_args = self.cmd_arg_parser.parse_args(cmd_args[1:])
-            self.primary_input_path = os.path.abspath(parsed_args.input_file)
+            args = cravat.util.get_args(self.cmd_arg_parser, inargs, inkwargs)
+            self.primary_input_path = os.path.abspath(args.input_file)
             self.secondary_paths = {}
-            if parsed_args.secondary_inputs:
-                for secondary_def in parsed_args.secondary_inputs:
-                    sec_name, sec_path = secondary_def.split('@')
+            if args.secondary_inputs:
+                for secondary_def in args.secondary_inputs:
+                    sec_name, sec_path = re.split(r'(?<!\\)=', secondary_def)
                     self.secondary_paths[sec_name] = os.path.abspath(sec_path)
             self.output_dir = os.path.dirname(self.primary_input_path)
-            if parsed_args.output_dir:
-                self.output_dir = parsed_args.output_dir
-
-            self.plain_output = parsed_args.plainoutput
-            self.output_basename = os.path.basename(self.primary_input_path)
-            if parsed_args.name:
-                self.output_basename = parsed_args.name
+            if args.output_dir:
+                self.output_dir = args.output_dir
+            self.plain_output = args.plainoutput
+            if hasattr(args, 'run_name') and args.run_name is not None:
+                self.output_basename = args.run_name
+            else:
+                self.output_basename = os.path.basename(self.primary_input_path)
+                if self.output_basename.endswith('.crx'):
+                    self.output_basename = self.output_basename[:-4]
             if self.output_basename != '__dummy__':
                 self.update_status_json_flag = True
             else:
                 self.update_status_json_flag = False
-            if parsed_args.conf:
-                self.job_conf_path = parsed_args.conf
+            if hasattr(args, 'conf'):
+                self.job_conf_path = args.conf
             self.confs = None
-            if parsed_args.confs is not None:
-                confs = parsed_args.confs.lstrip('\'').rstrip('\'').replace("'", '"')
+            if hasattr(args, 'confs') and args.confs is not None:
+                confs = args.confs.lstrip('\'').rstrip('\'').replace("'", '"')
                 self.confs = json.loads(confs)
+            self.args = args
         except Exception as e:
             self._log_exception(e)
 
     # Runs the annotator.
     def run(self):
-        if self.update_status_json_flag:
+        if self.update_status_json_flag and self.status_writer is not None:
             self.status_writer.queue_status_update('status', 'Started {} ({})'.format(self.conf['title'], self.module_name))
         try:
             start_time = time.time()
             self.logger.info('started: %s'%time.asctime(time.localtime(start_time)))
-            print('        {}: started at {}'.format(self.module_name, time.asctime(time.localtime(start_time))))
+            if self.args.silent == False:
+                print('        {}: started at {}'.format(self.module_name, time.asctime(time.localtime(start_time))))
             self.base_setup()
             last_status_update_time = time.time()
             for lnum, line, input_data, secondary_data in self._get_input():
                 try:
-                    if self.update_status_json_flag:
+                    if self.update_status_json_flag and self.status_writer is not None:
                         cur_time = time.time()
                         if lnum % 10000 == 0 or cur_time - last_status_update_time > 3:
                             self.status_writer.queue_status_update('status', 'Running {} ({}): line {}'.format(self.conf['title'], self.module_name, lnum))
                             last_status_update_time = cur_time
-                    if secondary_data == {}:
+                    if input_data.get('alt_base', '') == '*': # VCF format * allele gets empty annotation.
+                        output_dict = {}
+                    elif secondary_data == {}:
                         output_dict = self.annotate(input_data)
                     else:
                         output_dict = self.annotate(input_data, secondary_data)
@@ -219,7 +240,7 @@ class BaseAnnotator(object):
             run_time = end_time - start_time
             self.logger.info('runtime: {0:0.3f}s'.format(run_time))
             print('        {}: runtime {:0.3f}s'.format(self.module_name, run_time))
-            if self.update_status_json_flag:
+            if self.update_status_json_flag and self.status_writer is not None:
                 version = self.conf.get('version', 'unknown')
                 self.status_writer.queue_status_update('status', 'Finished {} ({})'.format(self.conf['title'], self.module_name))
         except Exception as e:
@@ -235,15 +256,15 @@ class BaseAnnotator(object):
         pass
 
     async def get_gene_summary_data (self, cf):
-        print('            {}: getting gene summary data'.format(self.module_name))
+        #print('            {}: getting gene summary data'.format(self.module_name))
         t = time.time()
-        hugos = await cf.get_filtered_hugo_list()
+        hugos = await cf.exec_db(cf.get_filtered_hugo_list)
         cols = [self.module_name + '__' + coldef['name'] \
             for coldef in self.conf['output_columns'] \
             if coldef['name'] != 'uid']
         data = {}
         t = time.time()
-        rows = await cf.get_variant_data_for_cols(cols)
+        rows = await cf.exec_db(cf.get_variant_data_for_cols, cols)
         rows_by_hugo = {}
         t = time.time()
         for row in rows:
@@ -259,7 +280,7 @@ class BaseAnnotator(object):
                 input_data[cols[i].split('__')[1]] = [row[i] for row in rows]
             out = self.summarize_by_gene(hugo, input_data)
             data[hugo] = out
-        print('            {}: finished getting gene summary data in {:0.3f}s'.format(self.module_name, time.time() - t))
+        #print('            {}: finished getting gene summary data in {:0.3f}s'.format(self.module_name, time.time() - t))
         return data
 
     def _log_runtime_exception (self, lnum, line, input_data, e):
