@@ -66,12 +66,8 @@ class MasterCravatConverter(object):
     """
     ALREADYCRV = 2
 
-    def __init__(self, args=None, status_writer=None):
-        args = args if args else sys.argv
-        self.status_writer = status_writer
-        self.input_paths = []
-        self.input_files = []
-        self.input_format = None
+    def __init__(self, *inargs, **inkwargs):
+        self._parse_cmd_args(inargs, inkwargs)
         self.logger = None
         self.crv_writer = None
         self.crs_writer = None
@@ -81,17 +77,12 @@ class MasterCravatConverter(object):
         self.converters = {}
         self.possible_formats = []
         self.ready_to_convert = False
-        self.cmd_args = None
-        self.output_dir = None
-        self.output_base_fname = None
-        self.pipeinput = False
         self.chromdict = {'chrx': 'chrX', 'chry': 'chrY', 'chrMT': 'chrM', 'chrMt': 'chrM', 'chr23': 'chrX', 'chr24': 'chrY'}
-        self._parse_cmd_args(args)
         self._setup_logger()
         self.vtracker = VTracker(deduplicate=not(self.unique_variants))
         self.wgsreader = cravat.get_wgs_reader(assembly='hg38')
 
-    def _parse_cmd_args(self, args):
+    def _parse_cmd_args(self, inargs, inkwargs):
         """ Parse the arguments in sys.argv """
         parser = argparse.ArgumentParser()
         parser.add_argument('path',
@@ -110,8 +101,8 @@ class MasterCravatConverter(object):
         parser.add_argument('-d', '--output-dir',
             dest='output_dir',
             help='Output directory. Default is input file directory.')
-        parser.add_argument('-l','--liftover',
-            dest='liftover',
+        parser.add_argument('-l','--genome',
+            dest='genome',
             choices=['hg38']+list(constants.liftover_chain_paths.keys()),
             default='hg38',
             help='Input gene assembly. Will be lifted over to hg38')
@@ -124,13 +115,16 @@ class MasterCravatConverter(object):
             default=False,
             action='store_true',
             help=argparse.SUPPRESS)
-        parsed_args = parser.parse_args(args)
+        parsed_args = cravat.util.get_args(parser, inargs, inkwargs)
+        self.input_format = None
         if parsed_args.format:
             self.input_format = parsed_args.format
         if parsed_args.inputs is None:
             raise ExpectedException('Input files are not given.')
+        self.pipeinput = False
         if parsed_args.inputs is not None and len(parsed_args.inputs) == 1 and parsed_args.inputs[0] == '-':
             self.pipeinput = True
+        self.input_paths = []
         if self.pipeinput == False:
             self.input_paths = [os.path.abspath(x) for x in parsed_args.inputs if x != '-']
         else:
@@ -145,33 +139,41 @@ class MasterCravatConverter(object):
         else:
             self.input_path_dict[0] = self.input_paths[0]
             self.input_path_dict2[STDIN] = 0
+        self.output_dir = None
         if parsed_args.output_dir:
             self.output_dir = parsed_args.output_dir
         else:
             self.output_dir = self.input_dir
         if not(os.path.exists(self.output_dir)):
             os.makedirs(self.output_dir)
+        self.output_base_fname = None
         if parsed_args.name:
             self.output_base_fname = parsed_args.name
         else:
             self.output_base_fname = os.path.basename(self.input_paths[0])
-        self.input_assembly = parsed_args.liftover
+        self.input_assembly = parsed_args.genome
         self.do_liftover = self.input_assembly != 'hg38'
         if self.do_liftover:
             self.lifter = LiftOver(constants.liftover_chain_paths[self.input_assembly])
         else:
             self.lifter = None
         self.status_fpath = os.path.join(self.output_dir, self.output_base_fname + '.status.json')
-        self.confs = None
+        self.conf = {}
         if parsed_args.confs is not None:
             confs = parsed_args.confs.lstrip('\'').rstrip('\'').replace("'", '"')
-            self.confs = json.loads(confs)
+            self.conf = json.loads(confs)
+        self.conf.update(parsed_args.conf)
         self.unique_variants = parsed_args.unique_variants
+        if parsed_args.status_writer:
+            self.status_writer = parsed_args.status_writer
+        else:
+            self.status_writer = None
 
     def setup (self):
         """ Do necesarry pre-run tasks """
         if self.ready_to_convert: return
         # Open file handle to input path
+        self.input_files = []
         if self.pipeinput == False:
             for input_path in self.input_paths:
                 encoding = detect_encoding(input_path)
@@ -264,6 +266,12 @@ class MasterCravatConverter(object):
         self.primary_converter = self.converters[self.input_format]
         self.primary_converter.output_dir = self.output_dir
         self.primary_converter.run_name = self.output_base_fname
+        self.primary_converter.input_assembly = self.input_assembly
+        module_name = self.primary_converter.format_name + '-converter'
+        if module_name in self.conf:
+            if hasattr(self.primary_converter, 'conf') == False:
+                self.primary_converter.conf = {}
+            self.primary_converter.conf.update(self.conf[module_name])
         if self.pipeinput == False:
             if len(self.input_files) > 1:
                 for f in self.input_files[1:]:
@@ -335,6 +343,7 @@ class MasterCravatConverter(object):
         fileno = 0
         total_lnum = 0
         base_re = re.compile('^[ATGC]+|[-]+$')
+        write_lnum = 0
         for f in self.input_files:
             if self.pipeinput == True:
                 fname = STDIN
@@ -345,7 +354,6 @@ class MasterCravatConverter(object):
             if self.pipeinput == False:
                 f.seek(0)
             read_lnum = 0
-            write_lnum = 0
             num_errors = 0
             if self.pipeinput:
                 cur_fname = STDIN
@@ -378,9 +386,6 @@ class MasterCravatConverter(object):
                                         wdict['sample_id'] = '__'.join([samp_prefix, wdict['sample_id']])
                                     else:
                                         wdict['sample_id'] = samp_prefix
-                                if self.do_liftover:
-                                    prelift_wdict = copy.copy(wdict)
-                                    wdict['chrom'], wdict['pos'] = self.liftover(wdict['chrom'], wdict['pos'])
                                 if 'ref_base' not in wdict or wdict['ref_base'] == '':
                                     wdict['ref_base'] = self.wgsreader.get_bases(chrom, int(wdict['pos']))
                                 else:
@@ -389,6 +394,9 @@ class MasterCravatConverter(object):
                                         raise BadFormatError('Reference base required for non SNV')
                                     elif ref_base is None or ref_base == '':
                                         wdict['ref_base'] = self.wgsreader.get_bases(chrom, int(pos))
+                                if self.do_liftover:
+                                    prelift_wdict = copy.copy(wdict)
+                                    wdict['chrom'], wdict['pos'] = self.liftover(wdict['chrom'], int(wdict['pos']), wdict['ref_base'], wdict['alt_base'])
                                 if base_re.fullmatch(wdict['ref_base']) is None:
                                     raise BadFormatError('Invalid reference base')
                                 if base_re.fullmatch(wdict['alt_base']) is None:
@@ -445,14 +453,66 @@ class MasterCravatConverter(object):
         self.status_writer.queue_status_update('status', 'Finished {} ({})'.format('Converter', self.primary_converter.format_name))
         return total_lnum, self.primary_converter.format_name
 
-    def liftover(self, old_chrom, old_pos):
-        new_coords = self.lifter.convert_coordinate(old_chrom, int(old_pos))
-        if new_coords != None and len(new_coords) > 0:
-            new_chrom = new_coords[0][0]
-            new_pos = new_coords[0][1]
-            return new_chrom, new_pos
+    def liftover(self, chrom, pos, ref, alt):
+        reflen = len(ref)
+        altlen = len(alt)
+        if reflen == 1 and altlen == 1:
+            res = self.lifter.convert_coordinate(chrom, pos - 1)
+            if res is None or len(res) == 0:
+                raise LiftoverFailure('Liftover failure')
+            if len(res) > 1:
+                raise LiftoverFailure('Liftover failure')
+            try:
+                el = res[0]
+            except:
+                raise LiftoverFailure('Liftover failure')
+            newchrom = el[0]
+            newpos = el[1] + 1
+        elif reflen >= 1 and altlen == 0: # del
+            pos1 = pos
+            pos2 = pos + reflen - 1
+            res1 = self.lifter.convert_coordinate(chrom, pos1 - 1)
+            res2 = self.lifter.convert_coordinate(chrom, pos2 - 1)
+            if res1 is None or res2 is None or len(res1) == 0 or len(res2) == 0:
+                raise LiftoverFailure('Liftover failure')
+            if len(res1) > 1 or len(res2) > 1:
+                raise LiftoverFailure('Liftover failure')
+            el1 = res1[0]
+            el2 = res2[0]
+            newchrom1 = el1[0]
+            newpos1 = el1[1] + 1
+            newchrom2 = el2[0]
+            newpos2 = el2[1] + 1
+            newchrom = newchrom1
+            newpos = newpos1
+            newpos = min(newpos1, newpos2)
+        elif reflen == 0 and altlen >= 1: # ins
+            res = self.lifter.convert_coordinate(chrom, pos - 1)
+            if res is None or len(res) == 0:
+                raise LiftoverFailure('Liftover failure')
+            if len(res) > 1:
+                raise LiftoverFailure('Liftover failure')
+            el = res[0]
+            newchrom = el[0]
+            newpos = el[1] + 1
         else:
-            raise LiftoverFailure('Liftover failure')
+            pos1 = pos
+            pos2 = pos + reflen - 1
+            res1 = self.lifter.convert_coordinate(chrom, pos1 - 1)
+            res2 = self.lifter.convert_coordinate(chrom, pos2 - 1)
+            if res1 is None or res2 is None or len(res1) == 0 or len(res2) == 0:
+                raise LiftoverFailure('Liftover failure')
+            if len(res1) > 1 or len(res2) > 1:
+                raise LiftoverFailure('Liftover failure')
+            el1 = res1[0]
+            el2 = res2[0]
+            newchrom1 = el1[0]
+            newpos1 = el1[1] + 1
+            newchrom2 = el2[0]
+            newpos2 = el2[1] + 1
+            newchrom = newchrom1
+            newpos = min(newpos1, newpos2)
+        return [newchrom, newpos]
 
     def _log_conversion_error(self, ln, line, e):
         """ Log exceptions thrown by primary converter.
