@@ -22,6 +22,9 @@ import cravat.cravat_class
 from types import SimpleNamespace
 import nest_asyncio
 nest_asyncio.apply()
+import sys
+if sys.platform == 'win32' and sys.version_info >= (3,8):
+    asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())
 
 class CravatReport:
 
@@ -39,10 +42,6 @@ class CravatReport:
         self.colname_conversion = {}
         self.warning_msgs = []
         self.parse_cmd_args(inargs, inkwargs)
-        if 'status_writer' not in inkwargs:
-            self.status_writer = None
-        else:
-            self.status_writer = status_writer
         global parser
         for ag in parser._action_groups:
             if ag.title == 'optional arguments':
@@ -109,6 +108,10 @@ class CravatReport:
                     parsed_args.inputfiles.append(input_path)
             c.close()
             db.close()
+        if hasattr(parsed_args, 'status_writer'):
+            self.status_writer = parsed_args.status_writer
+        else:
+            self.status_writer = None
         self.args = parsed_args
 
     async def prep (self):
@@ -407,8 +410,6 @@ class CravatReport:
                 await self.make_col_info(tab)
             await self.run_level(tab)
         await self.close_db()
-        await self.cf.close_db()
-        self.cf = None
         if self.module_conf is not None and self.status_writer is not None:
             if self.parsed_args.do_not_change_status == False:
                 self.status_writer.queue_status_update('status', 'Finished {} ({})'.format(self.module_conf['title'], self.module_name))
@@ -658,7 +659,9 @@ class CravatReport:
             exit()
 
     async def close_db (self):
-        pass
+        if self.cf is not None:
+            await self.cf.close_db()
+            self.cf = None
 
     async def load_filter (self):
         self.cf = await CravatFilter.create(dbpath=self.dbpath)
@@ -698,6 +701,15 @@ def run_reporter (*inargs, **inkwargs):
     args = cravat.util.get_args(parser, inargs, inkwargs)
     global au
     dbpath = args.dbpath
+    # Check if exists
+    if not os.path.exists(dbpath):
+        exit(f'{dbpath} not found')
+    # Check if database
+    try:
+        with sqlite3.connect(dbpath) as db:
+            db.execute('select * from info')
+    except:
+        exit(f'{dbpath} is not an OC database')
     compatible_version, db_version, oc_version = util.is_compatible_version(dbpath)
     if not compatible_version:
         if args.silent == False:
@@ -729,10 +741,17 @@ def run_reporter (*inargs, **inkwargs):
         args.do_not_change_status = True
         reporter = module.Reporter(args)
         loop.run_until_complete(reporter.prep())
-        response_t = loop.run_until_complete(reporter.run())
+        try:
+            response_t = loop.run_until_complete(reporter.run())
+            if args.silent == False:
+                print(f'report created in {os.path.abspath(output_dir)}.')
+        except:
+            if args.silent == False:
+                print(f'report generation failed.')
+            loop.run_until_complete(reporter.close_db())
+            response_t = {'success': False}
+            raise
         response[report_type] = response_t
-        if args.silent == False:
-            print(f'report created in {os.path.abspath(output_dir)}.')
     return response
 
 def cravat_report_entrypoint ():
