@@ -1,4 +1,3 @@
-#try:
 from http.server import HTTPServer, CGIHTTPRequestHandler
 from socketserver import TCPServer
 import os
@@ -175,6 +174,9 @@ def run (args):
             port = server.get('port')
             if args.result:
                 dbpath = args.result
+                if os.path.exists(dbpath) == False:
+                    print(f'{dbpath} does not exist. Exiting.')
+                    return
                 compatible_version, db_version, oc_version = cravat.util.is_compatible_version(dbpath)
                 if not compatible_version:
                     print(f'DB version {db_version} of {dbpath} is not compatible with the current OpenCRAVAT ({oc_version}).')
