@@ -539,6 +539,7 @@ def get_colmodel (tab, colinfo):
                     "listeners":["keyup"]}
                 column['retfilt'] = True
                 column['retfilttype'] = 'between'
+                column['dataType'] = 'float'
                 column['multiseloptions'] = []
             if 'col_genesummary' in d and d['col_genesummary'] == True:
                 genesummary_present = True
@@ -566,8 +567,12 @@ async def get_colinfo (dbpath, confpath, filterstring):
         await reporter.prep()
         colinfo = await reporter.get_variant_colinfo()
         await reporter.close_db()
+        if reporter.cf is not None:
+            await reporter.cf.close_db()
     except:
         await reporter.close_db()
+        if reporter.cf is not None:
+            await reporter.cf.close_db()
         raise
     return colinfo
 
@@ -626,6 +631,8 @@ async def serve_runwidget_post (request):
             pass
         else:
             val = '"' + val + '"'
+        if sys.platform == 'win32':
+            val = val.replace('\\', '\\\\')
         val = json.loads(val)
         new_queries[k] = val
     queries = new_queries
